let a = 6378245.0;
let ee = 0.00669342162296594323;
let pi = Math.PI
// 判断坐标点是否在中国境内
function outOfChina(lng, lat) {
	return (lng < 72.004 || lng > 137.8347 || lat < 0.8293 || lat > 55.8271);
}
// 转换经度的函数
function transformLon(lng, lat) {
	let ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + 0.1 * lng * lat + 0.1 * Math.sqrt(Math.abs(lng));
	ret += (20.0 * Math.sin(6.0 * lng * Math.PI / 180.0) + 20.0 * Math.sin(2.0 * lng * Math.PI / 180.0)) * 2.0 / 3.0;
	ret += (20.0 * Math.sin(lng * Math.PI / 180.0) + 40.0 * Math.sin(lng / 3.0 * Math.PI / 180.0)) * 2.0 / 3.0;
	ret += (150.0 * Math.sin(lng / 12.0 * Math.PI / 180.0) + 300.0 * Math.sin(lng / 30.0 * Math.PI / 180.0)) * 2.0 /
		3.0;
	return ret * (a / Math.sqrt(1 - ee * Math.sin(lat * Math.PI / 180.0) * Math.sin(lat * Math.PI / 180.0))) / 100000.0;
}

// 转换纬度的函数
function transformLat(lng, lat) {
	let ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + 0.1 * lng * lat + 0.2 * Math.sqrt(Math.abs(lng));
	ret += (20.0 * Math.sin(6.0 * lat * Math.PI / 180.0) + 20.0 * Math.sin(2.0 * lat * Math.PI / 180.0)) * 2.0 / 3.0;
	ret += (20.0 * Math.sin(lat * Math.PI / 180.0) + 40.0 * Math.sin(lat / 3.0 * Math.PI / 180.0)) * 2.0 / 3.0;
	ret += (160.0 * Math.sin(lat / 12.0 * Math.PI / 180.0) + 320.0 * Math.sin(lat / 30.0 * Math.PI / 180.0)) * 2.0 /
		3.0;
	return ret * (a * (1 - ee)) / Math.sqrt((1 - ee * Math.sin(lat * Math.PI / 180.0) * Math.sin(lat * Math.PI /
		180.0)) * (1 - ee * Math.sin(lat * Math.PI / 180.0) * Math.sin(lat * Math.PI / 180.0))) / 100000.0;
}

// WGS84转GCJ02的主函数
export const wgs84ToGCJ02 = (lng, lat) => {
	if (outOfChina(lng, lat)) {
		return { lng: lng, lat: lat };
	}
	let dLat = transformLat(lng - 105.0, lat - 35.0);
	let dLon = transformLon(lng - 105.0, lat - 35.0);
	let radLat = lat / 180.0 * Math.PI;
	let magic = Math.sin(radLat);
	magic = 1 - ee * magic * magic;
	let sqrtMagic = Math.sqrt(magic);
	dLat = (dLat * 180.0) / ((a * (1 - ee)) / (magic * sqrtMagic) * Math.PI );
	dLon = (dLon * 180.0) / (a / sqrtMagic * Math.PI * Math.cos(radLat));
	return {
		lng: lng + dLon,
		lat: lat + dLat
	};
}